import { useContext, useEffect, useState } from "react";
import { CalendarContext } from "./Calendar";
import styles from "./MonthCalendar.module.scss";
import {
  addMonths,
  differenceInWeeks,
  isSameMonth,
  startOfMonth,
  subMonths,
} from "date-fns";
import { daysOfWeek, monthNameOf, weekBoundsOfMonth } from "lib/date";
import Month from "./Month";
import ScrollContainer from "lib/ScrollContainer";

const NUM_MONTHS_SHOWN = 5;

export default function MonthCalendar() {
  const now = new Date();
  const { onDateClick, setCurrentTitle, viewportHeight } = useContext(
    CalendarContext
  );

  const range = [-2, -1, 0, 1, 2];
  const [monthsShown, setMonthsShown] = useState(
    range.map((x) => addMonths(startOfMonth(now), x))
  );
  const centerMonth = monthsShown[2];
  console.log(
    monthNameOf(centerMonth),
    monthsShown.map((x) => monthNameOf(x))
  );

  useEffect(() => {
    setCurrentTitle(
      <>
        <b>{monthNameOf(centerMonth)}</b>
        {centerMonth.getFullYear()}
      </>
    );
  }, [centerMonth]);

  const generatePreviousObject = (el: HTMLElement) => {
    const date = new Date(el.dataset.isodate!);
    const prevMonth = subMonths(date, 1);
    const newWeeksShown = [prevMonth, ...monthsShown];
    setMonthsShown(newWeeksShown.slice(0, NUM_MONTHS_SHOWN));
  };

  const generateNextObject = (el: HTMLElement) => {
    const date = new Date(el.dataset.isodate!);
    const nextMonth = addMonths(date, 1);
    const newWeeksShown = [...monthsShown, nextMonth];
    setMonthsShown(newWeeksShown.slice(-NUM_MONTHS_SHOWN));
  };

  const keyOf = (month: Date) => month.toISOString();

  // The calendar should always show 6 rows
  const dateCellHeight = viewportHeight / 6;

  const renderItem = (month: Date) => {
    // How many rows will this month take up?
    const [startWeek, endWeek] = weekBoundsOfMonth(month);
    const numWeeks = differenceInWeeks(endWeek, startWeek);
    const dateGridHeight = dateCellHeight * numWeeks;

    const props = {
      month,
      dateGridHeight,
      dateCellHeight,
      isActive: isSameMonth(centerMonth, month),
    };
    return (
      <Month key={month.toString()} {...props} onDateClick={onDateClick} />
    );
  };

  return (
    <>
      <div className={styles.daysOfWeek}>
        {daysOfWeek.map((day) => (
          <div key={day} className={styles.dayOfWeek}>
            {day}
          </div>
        ))}
      </div>

      <div className={styles.monthContainer}>
        <ScrollContainer
          className={styles.weekScroll}
          direction="vertical"
          list={monthsShown}
          renderItem={renderItem}
          keyOf={keyOf}
          generateNextObject={generateNextObject}
          generatePreviousObject={generatePreviousObject}
        />
      </div>
    </>
  );
}
