import { useContext, useEffect, useState } from "react";
import { CalendarContext } from "./Calendar";
import ScrollContainer from "lib/ScrollContainer";
import { addWeeks, getWeek, startOfWeek, subWeeks } from "date-fns";
import Week from "./Week";
import styles from "./WeekCalendar.module.scss";

const NUM_WEEKS_SHOWN = 5;

export default function WeekCalendar() {
  const now = new Date();
  const { onDateClick, setCurrentTitle } = useContext(CalendarContext);

  const range = [-2, -1, 0, 1, 2];
  const [weeksShown, setWeeksShown] = useState(
    range.map((x) => addWeeks(startOfWeek(now), x))
  );
  const centerWeek = weeksShown[2];

  useEffect(() => {
    setCurrentTitle(
      <>
        {centerWeek.getFullYear()}

        <b>{JSON.stringify(weeksShown.map((x) => getWeek(x)))}</b>
      </>
    );
  }, [centerWeek]);

  const generatePreviousObject = (el: HTMLElement) => {
    const date = new Date(el.dataset.isodate!);
    const prevWeek = subWeeks(date, 1);
    const newWeeksShown = [prevWeek, ...weeksShown];
    setWeeksShown(newWeeksShown.slice(0, NUM_WEEKS_SHOWN));
  };

  const generateNextObject = (el: HTMLElement) => {
    const date = new Date(el.dataset.isodate!);
    const nextWeek = addWeeks(date, 1);
    const newWeeksShown = [...weeksShown, nextWeek];
    setWeeksShown(newWeeksShown.slice(-NUM_WEEKS_SHOWN));
  };

  const keyOf = (week: Date) => week.toISOString();

  const renderItem = (week: Date) => {
    return <Week key={week.toISOString()} week={week} />;
  };

  return (
    <div className={styles.weekContainer}>
      <ScrollContainer
        className={styles.weekScroll}
        direction="horizontal"
        list={weeksShown}
        renderItem={renderItem}
        keyOf={keyOf}
        generateNextObject={generateNextObject}
        generatePreviousObject={generatePreviousObject}
      />
    </div>
  );
}
