import classNames from "classnames";
import styles from "./ToggleSwitch.module.scss";
import { Key } from "react";

interface ToggleSwitchProps<T> {
  options: T[];
  defaultOption?: string;
  value: T;
  renderValue: (_: T) => JSX.Element;
  setValue: (_: T) => any;
}

export default function ToggleSwitch<T extends Key>({
  options,
  defaultOption,
  value,
  renderValue,
  setValue,
}: ToggleSwitchProps<T>) {
  return (
    <div className={styles.toggleSwitch}>
      {options.map((option) => {
        const isSelected = option === value;
        return (
          <button
            key={option}
            className={classNames(styles.option, isSelected && styles.selected)}
            onClick={() => setValue(option)}
          >
            {renderValue(option)}
          </button>
        );
      })}
    </div>
  );
}
